package com.danmassey.rugbe.robot;

import com.danmassey.rugbe.board.Pkg;

/**
 * Represents my robot. Delegates position to an NBot.
 * 
 * @author dan
 * Created on Aug 30, 2002
 **/
public class Robot {
	private int id;
	private int capacity;
	private int money;
	private NBot nbot;
	private Pkg packages = null;
	private int carrying = 0;
	
	/**
	 * Constructor Robot.
	 * @param id
	 * @param capacity
	 * @param money
	 **/
	public Robot(int id, int capacity, int money) {
		this.id = id;
		this.capacity = capacity;
		this.money = money;
	}
	
	/**
	 * Can only be set once.
	 **/
	public void setNbot(NBot nbot) {
		if (this.nbot == null) {
			this.nbot = nbot;
		}
	}
	
	public int getId() {
		return this.id;
	}
	
	public void setId(int id) {
		this.id = id;
	}
	
	public int getCapacity() {
		return this.capacity;
	}
	
	public void setCapacity(int capacity) {
		this.capacity = capacity;
	}
	
	public int getMoney() {
		return this.money;
	}
	
	public void setMondey(int money) {
		this.money = money;
	}
	
	public int getX() {
		return this.nbot.getX();
	}
	
	public int getY() {
		return this.nbot.getY();
	}
	
	/**
	 * Method pickUp.
	 * @param pickPkgId
	 * @param packages
	 */
	public void pickUp(int pickPkgId, Pkg[] packages) {
		int i = 0;
		while (pickPkgId != packages[i].getId()) {
			i++;				
		}
		int newDistance = getPkgDistance(packages[i]);
		if (this.packages == null) {
			this.packages = packages[i];
		} else if (newDistance < getPkgDistance(this.packages)) {
			packages[i].setNextPkg(this.packages);
			this.packages = packages[i];
		} else {
			Pkg pkg = this.packages;
			while ((pkg.getNextPkg() != null) && (newDistance >= getPkgDistance(pkg))) {
				pkg = pkg.getNextPkg();
			}
			packages[i].setNextPkg(pkg.getNextPkg());
			pkg.setNextPkg(packages[i]);
		}
		this.carrying += packages[i].getWeight();
	}

	/**
	 * Method drop.
	 * @param dropPkgId
	 */
	public void drop(int dropPkgId) {
		if (this.packages.getId() == dropPkgId) {
			this.carrying -= this.packages.getWeight();
			this.packages = this.packages.getNextPkg();
		} else {
			Pkg pkg = this.packages;
			while (pkg.getNextPkg().getId() != dropPkgId) {
				pkg = pkg.getNextPkg();
			}
			this.carrying -= pkg.getNextPkg().getWeight();
			pkg.setNextPkg(pkg.getNextPkg().getNextPkg());
		}
	}
	
	public int getCarrying() {
		return this.carrying;
	}

	public Pkg getPackages() {
		return this.packages;
	}
	
	public int getPkgDistance(Pkg pkg) {
		int dX = Math.abs(getX() - pkg.getDestX());
		int dY = Math.abs(getY() - pkg.getDestY());
		return dX + dY;
	}
	
	public String toString() {
		StringBuffer buffy = new StringBuffer();
		buffy.append("Robot: ");
		buffy.append(this.id);
		buffy.append("  Capacity: ");
		buffy.append(this.capacity);
		buffy.append("  Money: ");
		buffy.append(this.money);
		return buffy.toString();
	}
}
